define([
    'jquery',
    'mage/url',
    'braintreeClient',
    'braintreeHostedFields'
], function ($, mageUrl, braintreeClient, braintreeHostedFields) {
    return {
        braintreeHostedFieldsInstance: null,
        form: document.getElementById('cc_form'),

        init: function (braintreeClientToken) {
            var loader = document.getElementById('cc_form_loader');
            $(loader).loader('show');

            var submit = document.getElementById('cc_form_submit_button');
            var ccNonce = document.getElementById('cc_nonce');
            var self = this;

            braintreeClient.create({
                authorization: braintreeClientToken
            }, function (clientErr, clientInstance) {
                if (clientErr) {
                    console.error(clientErr);
                    return;
                }

                braintreeHostedFields.create({
                    client: clientInstance,
                    styles: {
                        'input': {
                            'font-size': '14px'
                        },
                        'input.invalid': {
                            'color': 'red'
                        },
                        'input.valid': {
                            'color': 'green'
                        }
                    },
                    fields: {
                        number: {
                            container: '#cc_number',
                            maskInput: true,
                            placeholder: '4111 1111 1111 1111'
                        },
                        cvv: {
                            container: '#cc_cvv',
                            maskInput: true,
                            placeholder: '123'
                        },
                        expirationDate: {
                            container: '#cc_expiration',
                            placeholder: '10/2024'
                        }
                    }
                }, function (hostedFieldsErr, hostedFieldsInstance) {
                    if (hostedFieldsErr) {
                        console.error(hostedFieldsErr);
                        alert(hostedFieldsErr.message);
                        return;
                    }

                    hostedFieldsInstance.addClass('number', 'input-text required-entry');

                    submit.removeAttribute('disabled');

                    self.form.addEventListener('submit', function (event) {
                        event.preventDefault();

                        hostedFieldsInstance.tokenize({
                            vault: true
                        }, function (tokenizeErr, payload) {
                            if (tokenizeErr) {
                                console.error(tokenizeErr);
                                alert(tokenizeErr.message);
                                return;
                            }

                            ccNonce.value = payload.nonce;
                            self.form.submit();
                        });
                    }, false);

                    self.braintreeHostedFieldsInstance = hostedFieldsInstance;

                    $(loader).loader('hide');
                });
            });
        },

        deinit: function () {
            var creditCardPublicHashInput = document.getElementById('cc_public_hash');
            creditCardPublicHashInput.removeAttribute('value');
            this.form.action = mageUrl.build('braintreevault/card/createAction');

            if (this.braintreeHostedFieldsInstance === null) {
                console.error('Hosted Fields are not initialized!');
                return;
            }

            this.braintreeHostedFieldsInstance.teardown(function (teardownErr) {
                if (teardownErr) {
                    console.error('Could not tear down Hosted Fields!');
                } else {
                    console.info('Hosted Fields has been torn down!');
                }
            });
        }
    }
});
